/*
 * Copyright (C) 2018 Gravity Engineering Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.graveng.rci.server;

import java.util.HashMap;

import com.graveng.rci.util.Logger;

/**
 * サーバ側で発生したイベントをクライアントに伝えるためのシングルトン
 */
public class RciListenerInvoker {
	/**
	 * 自身のインスタンス
	 */
	protected static RciListenerInvoker own = new RciListenerInvoker();

	/**
	 * リスナー遠隔起動用の情報
	 * key:リスナのハッシュコード, value:リスナのInvoker
	 */
	protected HashMap<Integer, RciServer.Invoker> listenerInvoker = new HashMap<Integer, RciServer.Invoker>();

	/**
	 * 外部から生成できないクラス
	 */
	private RciListenerInvoker() {
	}

	/**
	 * インスタンスを取得する。
	 * @return RciListenerInvoker
	 */
	public static RciListenerInvoker gi() {
		return own;
	}

	/**
	 * RciServerがlistenerInvokerを取得する。
	 * @return listenerInvoker
	 */
	HashMap<Integer, RciServer.Invoker> getListenerInvoker() {
		return listenerInvoker;
	}

	/**
	 * 登録されているリスナー数を取得する。
	 * @return リスナー数
	 */
	public int getListenerCount() {
		return listenerInvoker.size();
	}

	/**
	 * クライアントにイベントを転送する。
	 * <p>
	 * 例)<br>
	 * 	public void keyPressed(java.awt.event.KeyEvent e) {<br>
	 * 		RciListenerInvoker.gi().invokeClientListener(hashCode(), "keyPressed", e);<br>
	 * 	}<br>
	 *
	 * @param listenerKey イベントクラスのhashcodeを設定する
	 * @param objects クライアントで起動する、メソッド名とそのパラメータ
	 * @return 転送を行った場合はtrue
	 */
	public boolean invokeClientListener(int listenerKey, Object ... objects ) {
		boolean result;
		try {
			RciServer.Invoker invoker = listenerInvoker.get(listenerKey);
			if(invoker == null) {
				// 登録されていない
				throw new NullPointerException("listenerKey not found:"+listenerKey);
			}
			invoker.invokeClientListener(listenerKey, objects);
			result = true;
		} catch(Exception e) {
			Logger.warn("callback failed", e);
			result = false;
		}
		return result;
	}
}
