/*
 * Copyright (C) 2018 Gravity Engineering Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.graveng.rci.util;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

/**
 * オブジェクトのファイル入出力を可能にする
 */
public class Serializer extends ObjectEx implements Serializable{

	// XMLのシリアライズはなぜかうまくいかない。
	// →全ての変更可能なフィールドにはsetter, getterを用意しなければならない。
	/**
	 * XML形式でオブジェクトのファイル入出力を行う場合はtrue、シリアライズする場合はfalseにする。
	 * （XMLの場合はpublicなsetter, getterを全てのフィールドに用意しなければならない。）
	 */
	public static boolean useXml = false;

	/**
	 * 継承しているクラスをファイルに保存する。
	 * @param filepath フルパス
	 * @throws IOException IO例外
	 */
	public void save(String filepath) throws IOException {
		if(useXml) {
			saveXML(filepath);
		}
		else {
			saveBinary(filepath);
		}
	}

	/**
	 * オブジェクトをファイルに保存する。
	 * @param filepath フルパス
	 * @param obj オブジェクト
	 * @throws IOException IO例外
	 */
	public static void save(String filepath, Serializer obj) throws IOException {
		if(useXml) {
			saveXML(filepath, obj);
		}
		else {
			saveBinary(filepath, obj);
		}
	}

	/**
	 * クラスをファイルからロードする。
	 * @param filepath フルパス
	 * @return Serializer
	 * @throws IOException IO例外
	 * @throws ClassNotFoundException クラスは存在しない
	 */
	public static Serializer load(String filepath) throws IOException, ClassNotFoundException {
		if(useXml) {
			return loadXML(filepath);
		}
		else {
			return loadBinary(filepath);
		}
	}

	/**
	 * 継承しているクラスをシリアライズしてファイルに保存する。
	 * @param filepath フルパス
	 * @throws IOException IO例外
	 */
	protected void saveBinary(String filepath) throws IOException {
		 ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(filepath));
	     oos.writeObject(this);
	     oos.close();
	}

	/**
	 * オブジェクトをシリアライズしてファイルに保存する。
	 * @param filepath フルパス
	 * @param obj オブジェクト
	 * @throws IOException IO例外
	 */
	protected static void saveBinary(String filepath, Serializer obj) throws IOException {
		 ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(filepath));
	     oos.writeObject(obj);
	     oos.close();
	}

	/**
	 * クラスをシリアライズしたファイルからロードする。
	 * @param filepath フルパス
	 * @return Serializer
	 * @throws IOException IO例外
	 * @throws ClassNotFoundException クラスは存在しない
	 */
	protected static Serializer loadBinary(String filepath) throws IOException, ClassNotFoundException {
		 ObjectInputStream ois = new ObjectInputStream(new FileInputStream(filepath));
		 Serializer o = (Serializer)ois.readObject();
		 ois.close();
		 return o;
	}

	/**
	 * クラスをシリアライズしたファイルからこのオブジェクトにロードする。
	 * @param filepath フルパス
	 * @throws IOException IO例外
	 */
	protected void loadBinaryOwn(String filepath) throws Exception {
		 ObjectInputStream ois = new ObjectInputStream(new FileInputStream(filepath));
		 Serializer o = (Serializer)ois.readObject();
		 ois.close();

		 ClassField.clone(o, this);
	}

	/**
	 * 継承しているクラスをXML形式でファイルに保存する。
	 * @param filepath フルパス
	 * @throws IOException IO例外
	 */
	public void saveXML(String filepath) throws IOException {
		XMLEncoder encoder = new XMLEncoder(
                new BufferedOutputStream(
                    new FileOutputStream(filepath)));
            encoder.writeObject(this);
            encoder.close();
	}

	/**
	 * オブジェクトをXML形式でファイルに保存する。
	 * @param filepath フルパス
	 * @param obj オブジェクト
	 * @throws IOException IO例外
	 */
	protected static void saveXML(String filepath, Serializer obj) throws IOException {
		XMLEncoder encoder = new XMLEncoder(
                new BufferedOutputStream(
                    new FileOutputStream(filepath)));
            encoder.writeObject(obj);
            encoder.close();
	}

	/**
	 * クラスをxmlファイルからオブジェクトにロードする。
	 * @param filepath フルパス
	 * @return Serializer
	 * @throws IOException IO例外
	 */
	public static Serializer loadXML(String filepath) throws IOException {
		XMLDecoder decoder = new XMLDecoder(
                new BufferedInputStream(
                    new FileInputStream(filepath)));
		Serializer o = (Serializer)decoder.readObject();
		decoder.close();
        return o;
	}

	/**
	 * クラスをxmlファイルからこのオブジェクトにロードする。
	 * @param filepath フルパス
	 * @throws IOException IO例外
	 */
	public void loadXMLOwn(String filepath) throws Exception {
		XMLDecoder decoder = new XMLDecoder(
                new BufferedInputStream(
                    new FileInputStream(filepath)));
		Serializer o = (Serializer)decoder.readObject();
		decoder.close();
		ClassField.clone(o, this);
	}

	// シリアライズ(Class, Field)を使えば、setter, getterが無くてもprivateのフィールド変数でも
	// 書き換えることが可能であり、個別のフィールド変数をXML化することで、
	// ObjectOutputStreamのように制約がなくXMLファイルへのEncode, Decodeが可能になる。
	/**
	 * ObjectをXML形式の文字列として返す。
	 *
	 * @param obj Object
	 * @return Objectのxml形式
	 * @throws IOException IO例外
	 */
	// Packetにも同じものがあるが、Packetでこのクラスは使わないようにした。
	public static String getEncodeXML(Object obj) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		XMLEncoder encoder = new XMLEncoder(baos);
        encoder.writeObject(obj);
        encoder.close();
        String result = baos.toString();
        return result;
	}

	/**
	 * XML形式の文字列からObjectを生成する。
	 *
	 * @param xmlObject Objectのxml形式
	 * @return Object
	 * @throws IOException IO例外
	 */
	// Packetにも同じものがあるが、Packetでこのクラスは使わないようにした。
	public static Object getDecodeXML(String xmlObject) throws IOException {
		ByteArrayInputStream baos = new ByteArrayInputStream(xmlObject.getBytes());
		XMLDecoder decoder = new XMLDecoder(baos);
        Object obj = decoder.readObject();
        decoder.close();
        return obj;
	}
}
